@@ -32,6 +32,7 @@ import org.apache.gravitino.Schema;
 import org.apache.gravitino.authorization.Group;
 import org.apache.gravitino.authorization.Owner;
 import org.apache.gravitino.authorization.Privilege;
+import org.apache.gravitino.authorization.Privileges;
 import org.apache.gravitino.authorization.Role;
 import org.apache.gravitino.authorization.SecurableObject;
 import org.apache.gravitino.authorization.User;
@@ -678,6 +679,32 @@ public class DTOConverters {
     return Arrays.stream(catalogs).map(DTOConverters::toDTO).toArray(CatalogDTO[]::new);
   }
 
+  /**
+   * Converts an array of Users to an array of UserDTOs.
+   *
+   * @param users The users to be converted.
+   * @return The array of UserDTOs.
+   */
+  public static UserDTO[] toDTOs(User[] users) {
+    if (ArrayUtils.isEmpty(users)) {
+      return new UserDTO[0];
+    }
+    return Arrays.stream(users).map(DTOConverters::toDTO).toArray(UserDTO[]::new);
+  }
+
+  /**
+   * Converts an array of Groups to an array of GroupDTOs.
+   *
+   * @param groups The groups to be converted.
+   * @return The array of GroupDTOs.
+   */
+  public static GroupDTO[] toDTOs(Group[] groups) {
+    if (ArrayUtils.isEmpty(groups)) {
+      return new GroupDTO[0];
+    }
+    return Arrays.stream(groups).map(DTOConverters::toDTO).toArray(GroupDTO[]::new);
+  }
+
   /**
    * Converts a DistributionDTO to a Distribution.
    *
@@ -977,4 +1004,18 @@ public class DTOConverters {
         throw new IllegalArgumentException("Unsupported partitioning: " + partitioning.strategy());
     }
   }
+
+  /**
+   * Converts a Privilege DTO to a Privilege
+   *
+   * @param privilegeDTO The privilege DTO to be converted.
+   * @return The privilege.
+   */
+  public static Privilege fromPrivilegeDTO(PrivilegeDTO privilegeDTO) {
+    if (privilegeDTO.condition().equals(Privilege.Condition.ALLOW)) {
+      return Privileges.allow(privilegeDTO.name());
+    } else {
+      return Privileges.deny(privilegeDTO.name());
+    }
+  }
 }
