@@ -24,6 +24,7 @@ import com.google.common.collect.Lists;
 import java.time.Instant;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.stream.Collectors;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.gravitino.Catalog;
@@ -48,6 +49,8 @@ import org.apache.gravitino.meta.TagEntity;
 import org.apache.gravitino.meta.TopicEntity;
 import org.apache.gravitino.meta.UserEntity;
 import org.apache.gravitino.storage.relational.po.CatalogPO;
+import org.apache.gravitino.storage.relational.po.ExtendedGroupPO;
+import org.apache.gravitino.storage.relational.po.ExtendedUserPO;
 import org.apache.gravitino.storage.relational.po.FilesetPO;
 import org.apache.gravitino.storage.relational.po.FilesetVersionPO;
 import org.apache.gravitino.storage.relational.po.GroupPO;
@@ -728,6 +731,56 @@ public class POConverters {
     }
   }
 
+  /**
+   * Convert {@link ExtendedUserPO} to {@link UserEntity}
+   *
+   * @param userPO ExtendedUserPO object to be converted
+   * @param namespace Namespace object to be associated with the user
+   * @return UserEntity object from ExtendedUserPO object
+   */
+  public static UserEntity fromExtendedUserPO(ExtendedUserPO userPO, Namespace namespace) {
+    try {
+      UserEntity.Builder builder =
+          UserEntity.builder()
+              .withId(userPO.getUserId())
+              .withName(userPO.getUserName())
+              .withNamespace(namespace)
+              .withAuditInfo(
+                  JsonUtils.anyFieldMapper().readValue(userPO.getAuditInfo(), AuditInfo.class));
+      if (StringUtils.isNotBlank(userPO.getRoleNames())) {
+        List<String> roleNamesFromJson =
+            JsonUtils.anyFieldMapper().readValue(userPO.getRoleNames(), List.class);
+        List<String> roleNames =
+            roleNamesFromJson.stream().filter(StringUtils::isNotBlank).collect(Collectors.toList());
+        if (!roleNames.isEmpty()) {
+          builder.withRoleNames(roleNames);
+        }
+      }
+
+      if (StringUtils.isNotBlank(userPO.getRoleIds())) {
+        // Different JSON AGG from backends will produce different types data, we
+        // can only use Object. PostSQL produces the data with type Long. H2 produces
+        // the data with type String.
+        List<Object> roleIdsFromJson =
+            JsonUtils.anyFieldMapper().readValue(userPO.getRoleIds(), List.class);
+        List<Long> roleIds =
+            roleIdsFromJson.stream()
+                .filter(Objects::nonNull)
+                .map(String::valueOf)
+                .filter(StringUtils::isNotBlank)
+                .map(Long::valueOf)
+                .collect(Collectors.toList());
+
+        if (!roleIds.isEmpty()) {
+          builder.withRoleIds(roleIds);
+        }
+      }
+      return builder.build();
+    } catch (JsonProcessingException e) {
+      throw new RuntimeException("Failed to deserialize json object:", e);
+    }
+  }
+
   /**
    * Convert {@link GroupPO} to {@link GroupEntity}
    *
@@ -762,6 +815,57 @@ public class POConverters {
     }
   }
 
+  /**
+   * Convert {@link ExtendedGroupPO} to {@link GroupEntity}
+   *
+   * @param groupPO ExtendedGroupPO object to be converted
+   * @param namespace Namespace object to be associated with the user
+   * @return GroupEntity object from ExtendedGroupPO object
+   */
+  public static GroupEntity fromExtendedGroupPO(ExtendedGroupPO groupPO, Namespace namespace) {
+    try {
+      GroupEntity.Builder builder =
+          GroupEntity.builder()
+              .withId(groupPO.getGroupId())
+              .withName(groupPO.getGroupName())
+              .withNamespace(namespace)
+              .withAuditInfo(
+                  JsonUtils.anyFieldMapper().readValue(groupPO.getAuditInfo(), AuditInfo.class));
+
+      if (StringUtils.isNotBlank(groupPO.getRoleNames())) {
+        List<String> roleNamesFromJson =
+            JsonUtils.anyFieldMapper().readValue(groupPO.getRoleNames(), List.class);
+        List<String> roleNames =
+            roleNamesFromJson.stream().filter(StringUtils::isNotBlank).collect(Collectors.toList());
+        if (!roleNames.isEmpty()) {
+          builder.withRoleNames(roleNames);
+        }
+      }
+
+      if (StringUtils.isNotBlank(groupPO.getRoleIds())) {
+        // Different JSON AGG from backends will produce different types data, we
+        // can only use Object. PostSQL produces the data with type Long. H2 produces
+        // the data with type String.
+        List<Object> roleIdsFromJson =
+            JsonUtils.anyFieldMapper().readValue(groupPO.getRoleIds(), List.class);
+        List<Long> roleIds =
+            roleIdsFromJson.stream()
+                .filter(Objects::nonNull)
+                .map(String::valueOf)
+                .filter(StringUtils::isNotBlank)
+                .map(Long::valueOf)
+                .collect(Collectors.toList());
+
+        if (!roleIds.isEmpty()) {
+          builder.withRoleIds(roleIds);
+        }
+      }
+      return builder.build();
+    } catch (JsonProcessingException e) {
+      throw new RuntimeException("Failed to deserialize json object:", e);
+    }
+  }
+
   /**
    * Initialize UserRoleRelPO
    *
@@ -965,6 +1069,27 @@ public class POConverters {
     }
   }
 
+  public static RolePO updateRolePOWithVersion(RolePO oldRolePO, RoleEntity newRole) {
+    Long lastVersion = oldRolePO.getLastVersion();
+    // TODO: set the version to the last version + 1 when having some fields need be multiple
+    // version
+    Long nextVersion = lastVersion;
+    try {
+      return RolePO.builder()
+          .withRoleId(oldRolePO.getRoleId())
+          .withRoleName(newRole.name())
+          .withMetalakeId(oldRolePO.getMetalakeId())
+          .withProperties(JsonUtils.anyFieldMapper().writeValueAsString(newRole.properties()))
+          .withAuditInfo(JsonUtils.anyFieldMapper().writeValueAsString(newRole.auditInfo()))
+          .withCurrentVersion(nextVersion)
+          .withLastVersion(nextVersion)
+          .withDeletedAt(DEFAULT_DELETED_AT)
+          .build();
+    } catch (JsonProcessingException e) {
+      throw new RuntimeException("Failed to serialize json object:", e);
+    }
+  }
+
   public static TagEntity fromTagPO(TagPO tagPO, Namespace namespace) {
     try {
       return TagEntity.builder()
