@@ -18,7 +18,6 @@
  */
 package org.apache.gravitino.tag;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
@@ -31,11 +30,11 @@ import java.util.Set;
 import org.apache.gravitino.Entity;
 import org.apache.gravitino.EntityAlreadyExistsException;
 import org.apache.gravitino.EntityStore;
-import org.apache.gravitino.GravitinoEnv;
 import org.apache.gravitino.MetadataObject;
 import org.apache.gravitino.NameIdentifier;
 import org.apache.gravitino.Namespace;
 import org.apache.gravitino.exceptions.NoSuchEntityException;
+import org.apache.gravitino.exceptions.NoSuchMetadataObjectException;
 import org.apache.gravitino.exceptions.NoSuchMetalakeException;
 import org.apache.gravitino.exceptions.NoSuchTagException;
 import org.apache.gravitino.exceptions.NotFoundException;
@@ -240,14 +239,11 @@ public class TagManager {
   }
 
   public Tag[] listTagsInfoForMetadataObject(String metalake, MetadataObject metadataObject)
-      throws NotFoundException {
+      throws NoSuchMetadataObjectException {
     NameIdentifier entityIdent = MetadataObjectUtil.toEntityIdent(metalake, metadataObject);
     Entity.EntityType entityType = MetadataObjectUtil.toEntityType(metadataObject);
 
-    if (!checkAndImportEntity(metalake, metadataObject, GravitinoEnv.getInstance())) {
-      throw new NotFoundException(
-          "Failed to list tags for metadata object %s due to not found", metadataObject);
-    }
+    MetadataObjectUtil.checkMetadataObject(metalake, metadataObject);
 
     return TreeLockUtils.doWithTreeLock(
         entityIdent,
@@ -258,7 +254,7 @@ public class TagManager {
                 .listAssociatedTagsForMetadataObject(entityIdent, entityType)
                 .toArray(new Tag[0]);
           } catch (NoSuchEntityException e) {
-            throw new NotFoundException(
+            throw new NoSuchMetadataObjectException(
                 e, "Failed to list tags for metadata object %s due to not found", metadataObject);
           } catch (IOException e) {
             LOG.error("Failed to list tags for metadata object {}", metadataObject, e);
@@ -268,15 +264,12 @@ public class TagManager {
   }
 
   public Tag getTagForMetadataObject(String metalake, MetadataObject metadataObject, String name)
-      throws NotFoundException {
+      throws NoSuchMetadataObjectException {
     NameIdentifier entityIdent = MetadataObjectUtil.toEntityIdent(metalake, metadataObject);
     Entity.EntityType entityType = MetadataObjectUtil.toEntityType(metadataObject);
     NameIdentifier tagIdent = ofTagIdent(metalake, name);
 
-    if (!checkAndImportEntity(metalake, metadataObject, GravitinoEnv.getInstance())) {
-      throw new NotFoundException(
-          "Failed to get tag for metadata object %s due to not found", metadataObject);
-    }
+    MetadataObjectUtil.checkMetadataObject(metalake, metadataObject);
 
     return TreeLockUtils.doWithTreeLock(
         entityIdent,
@@ -289,7 +282,7 @@ public class TagManager {
               throw new NoSuchTagException(
                   e, "Tag %s does not exist for metadata object %s", name, metadataObject);
             } else {
-              throw new NotFoundException(
+              throw new NoSuchMetadataObjectException(
                   e, "Failed to get tag for metadata object %s due to not found", metadataObject);
             }
           } catch (IOException e) {
@@ -301,20 +294,18 @@ public class TagManager {
 
   public String[] associateTagsForMetadataObject(
       String metalake, MetadataObject metadataObject, String[] tagsToAdd, String[] tagsToRemove)
-      throws NotFoundException, TagAlreadyAssociatedException {
+      throws NoSuchMetadataObjectException, TagAlreadyAssociatedException {
     Preconditions.checkArgument(
         !metadataObject.type().equals(MetadataObject.Type.METALAKE)
-            && !metadataObject.type().equals(MetadataObject.Type.COLUMN),
+            && !metadataObject.type().equals(MetadataObject.Type.COLUMN)
+            && !metadataObject.type().equals(MetadataObject.Type.ROLE),
         "Cannot associate tags for unsupported metadata object type %s",
         metadataObject.type());
 
     NameIdentifier entityIdent = MetadataObjectUtil.toEntityIdent(metalake, metadataObject);
     Entity.EntityType entityType = MetadataObjectUtil.toEntityType(metadataObject);
 
-    if (!checkAndImportEntity(metalake, metadataObject, GravitinoEnv.getInstance())) {
-      throw new NotFoundException(
-          "Failed to associate tags for metadata object %s due to not found", metadataObject);
-    }
+    MetadataObjectUtil.checkMetadataObject(metalake, metadataObject);
 
     // Remove all the tags that are both set to add and remove
     Set<String> tagsToAddSet = tagsToAdd == null ? Sets.newHashSet() : Sets.newHashSet(tagsToAdd);
@@ -347,7 +338,7 @@ public class TagManager {
                         .map(Tag::name)
                         .toArray(String[]::new);
                   } catch (NoSuchEntityException e) {
-                    throw new NotFoundException(
+                    throw new NoSuchMetadataObjectException(
                         e,
                         "Failed to associate tags for metadata object %s due to not found",
                         metadataObject);
@@ -425,33 +416,4 @@ public class TagManager {
                 .build())
         .build();
   }
-
-  // This method will check if the entity is existed explicitly, internally this check will load
-  // the entity from underlying sources to entity store if not stored, and will allocate an uid
-  // for this entity, with this uid tags can be associated with this entity.
-  // This method should be called out of the tree lock, otherwise it will cause deadlock.
-  @VisibleForTesting
-  boolean checkAndImportEntity(String metalake, MetadataObject metadataObject, GravitinoEnv env) {
-    NameIdentifier entityIdent = MetadataObjectUtil.toEntityIdent(metalake, metadataObject);
-    Entity.EntityType entityType = MetadataObjectUtil.toEntityType(metadataObject);
-
-    switch (entityType) {
-      case METALAKE:
-        return env.metalakeDispatcher().metalakeExists(entityIdent);
-      case CATALOG:
-        return env.catalogDispatcher().catalogExists(entityIdent);
-      case SCHEMA:
-        return env.schemaDispatcher().schemaExists(entityIdent);
-      case TABLE:
-        return env.tableDispatcher().tableExists(entityIdent);
-      case TOPIC:
-        return env.topicDispatcher().topicExists(entityIdent);
-      case FILESET:
-        return env.filesetDispatcher().filesetExists(entityIdent);
-      case COLUMN:
-      default:
-        throw new IllegalArgumentException(
-            "Unsupported metadata object type: " + metadataObject.type());
-    }
-  }
 }
