@@ -27,6 +27,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.time.Instant;
+import java.util.Comparator;
 import java.util.List;
 import java.util.Optional;
 import java.util.function.Function;
@@ -119,6 +120,77 @@ class TestGroupMetaService extends TestJDBCBackend {
         Sets.newHashSet(group2.roleNames()), Sets.newHashSet(actualGroup.roleNames()));
   }
 
+  @Test
+  void testListGroups() throws IOException {
+    AuditInfo auditInfo =
+        AuditInfo.builder().withCreator("creator").withCreateTime(Instant.now()).build();
+    BaseMetalake metalake =
+        createBaseMakeLake(RandomIdGenerator.INSTANCE.nextId(), metalakeName, auditInfo);
+    backend.insert(metalake, false);
+
+    CatalogEntity catalog =
+        createCatalog(
+            RandomIdGenerator.INSTANCE.nextId(), Namespace.of(metalakeName), "catalog", auditInfo);
+    backend.insert(catalog, false);
+
+    GroupEntity group1 =
+        createGroupEntity(
+            RandomIdGenerator.INSTANCE.nextId(),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
+            "group1",
+            auditInfo);
+
+    RoleEntity role1 =
+        createRoleEntity(
+            RandomIdGenerator.INSTANCE.nextId(),
+            AuthorizationUtils.ofRoleNamespace("metalake"),
+            "role1",
+            auditInfo,
+            "catalog");
+    backend.insert(role1, false);
+
+    RoleEntity role2 =
+        createRoleEntity(
+            RandomIdGenerator.INSTANCE.nextId(),
+            AuthorizationUtils.ofRoleNamespace("metalake"),
+            "role2",
+            auditInfo,
+            "catalog");
+    backend.insert(role2, false);
+
+    GroupEntity group2 =
+        createGroupEntity(
+            RandomIdGenerator.INSTANCE.nextId(),
+            AuthorizationUtils.ofGroupNamespace("metalake"),
+            "group2",
+            auditInfo,
+            Lists.newArrayList(role1.name(), role2.name()),
+            Lists.newArrayList(role1.id(), role2.id()));
+
+    backend.insert(group1, false);
+    backend.insert(group2, false);
+
+    GroupMetaService groupMetaService = GroupMetaService.getInstance();
+    List<GroupEntity> actualGroups =
+        groupMetaService.listGroupsByNamespace(
+            AuthorizationUtils.ofGroupNamespace(metalakeName), true);
+    actualGroups.sort(Comparator.comparing(GroupEntity::name));
+    List<GroupEntity> expectGroups = Lists.newArrayList(group1, group2);
+    Assertions.assertEquals(expectGroups.size(), actualGroups.size());
+    for (int index = 0; index < expectGroups.size(); index++) {
+      Assertions.assertEquals(expectGroups.get(index).name(), actualGroups.get(index).name());
+      if (expectGroups.get(index).roleNames() == null) {
+        Assertions.assertNull(actualGroups.get(index).roleNames());
+      } else {
+        Assertions.assertEquals(
+            expectGroups.get(index).roleNames().size(), actualGroups.get(index).roleNames().size());
+        for (String roleName : expectGroups.get(index).roleNames()) {
+          Assertions.assertTrue(actualGroups.get(index).roleNames().contains(roleName));
+        }
+      }
+    }
+  }
+
   @Test
   void insertGroup() throws IOException {
     AuditInfo auditInfo =
@@ -243,7 +315,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group3Overwrite =
         createGroupEntity(
             group1.id(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group3Overwrite",
             auditInfo,
             Lists.newArrayList(role3.name()),
@@ -260,7 +332,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group4Overwrite =
         createGroupEntity(
             group1.id(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group4Overwrite",
             auditInfo);
     Assertions.assertDoesNotThrow(() -> groupMetaService.insertGroup(group4Overwrite, true));
@@ -779,7 +851,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group1 =
         createGroupEntity(
             RandomIdGenerator.INSTANCE.nextId(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group1",
             auditInfo,
             Lists.newArrayList(role1.name(), role2.name()),
@@ -787,7 +859,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group2 =
         createGroupEntity(
             RandomIdGenerator.INSTANCE.nextId(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group2",
             auditInfo,
             Lists.newArrayList(role1.name(), role2.name()),
@@ -795,7 +867,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group3 =
         createGroupEntity(
             RandomIdGenerator.INSTANCE.nextId(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group3",
             auditInfo,
             Lists.newArrayList(role1.name(), role2.name()),
@@ -803,7 +875,7 @@ class TestGroupMetaService extends TestJDBCBackend {
     GroupEntity group4 =
         createGroupEntity(
             RandomIdGenerator.INSTANCE.nextId(),
-            AuthorizationUtils.ofUserNamespace(metalakeName),
+            AuthorizationUtils.ofGroupNamespace(metalakeName),
             "group4",
             auditInfo,
             Lists.newArrayList(role1.name(), role2.name()),
