@@ -143,6 +143,11 @@ public class ExceptionHandlers {
     return OwnerExceptionHandler.INSTANCE.handle(type, name, metalake, e);
   }
 
+  public static Response handleRolePermissionOperationException(
+      OperationType type, String name, String parent, Exception e) {
+    return RolePermissionOperationHandler.INSTANCE.handle(type, name, parent, e);
+  }
+
   private static class PartitionExceptionHandler extends BaseExceptionHandler {
 
     private static final ExceptionHandler INSTANCE = new PartitionExceptionHandler();
@@ -508,6 +513,19 @@ public class ExceptionHandlers {
     }
   }
 
+  private static class RolePermissionOperationHandler extends BasePermissionExceptionHandler {
+
+    private static final ExceptionHandler INSTANCE = new RolePermissionOperationHandler();
+
+    @Override
+    protected String getPermissionErrorMsg(
+        String object, String operation, String parent, String reason) {
+      return String.format(
+          "Failed to operate object(%s) operation [%s] under role [%s], reason [%s]",
+          object, operation, parent, reason);
+    }
+  }
+
   private abstract static class BasePermissionExceptionHandler extends BaseExceptionHandler {
 
     protected abstract String getPermissionErrorMsg(
